﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;

namespace ArduinoGrapher
{
    public partial class Form1 : Form
    {
        //Global variables
        string fileName = "";

        public Form1()
        {
            InitializeComponent();
        }

        private void btnLoadLog_Click(object sender, EventArgs e)
        {
            //Create an instance of the dialog box
            OpenFileDialog openFileDialog1 = new OpenFileDialog();

            //Process input
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    fileName = openFileDialog1.FileName;

                    lblFilename.Text = fileName;

                    BuildChart();
                    UpdateGraphs(fileName);

                    btnUpdate.Enabled = true;
                }
                catch
                {
                    MessageBox.Show("There was a problem opening and processing the log file.", "Error");
                }
            }
        }

        void BuildChart()
        {
            //First, let's make sure the values we will be playing with are valid
            if (IsTextboxDouble(txtXMin) == false) return;
            if (IsTextboxDouble(txtXMax) == false) return;
            if (IsTextboxDouble(txtXInterval) == false) return;
            if (IsTextboxDouble(txtYMinimum) == false) return;
            if (IsTextboxDouble(txtYMax) == false) return;
            if (IsTextboxDouble(txtYInterval) == false) return;
            if (IsTextboxDouble(txtSeriesOneFactor) == false) return;
            if (IsTextboxDouble(txtSeries2Factor) == false) return;
            if (IsTextboxDouble(txtSeries3Factor) == false) return;
            if (IsTextboxDouble(txtSeries4Factor) == false) return;

            //Setup the X axis
            chart1.ChartAreas[0].AxisX.Minimum = Convert.ToInt32(txtXMin.Text);
            chart1.ChartAreas[0].AxisX.Maximum = Convert.ToInt32(txtXMax.Text);

            chart1.ChartAreas[0].AxisX.Interval = Convert.ToInt32(txtXInterval.Text);

            chart1.ChartAreas[0].AxisX.MajorGrid.LineColor = Color.Gainsboro;
            chart1.ChartAreas[0].AxisX.Title = "Record Number";

            chart1.ChartAreas[0].AxisX.LabelStyle.Angle = -90;      //Make the X axis type appear vertically

            //Setup the Y axis
            chart1.ChartAreas[0].AxisY.Minimum = Convert.ToInt32(txtYMinimum.Text);
            chart1.ChartAreas[0].AxisY.Maximum = Convert.ToInt32(txtYMax.Text);

            chart1.ChartAreas[0].AxisY.Interval = Convert.ToInt32(txtYInterval.Text);

            chart1.ChartAreas[0].AxisY.MajorGrid.LineColor = Color.Gainsboro;
            chart1.ChartAreas[0].AxisY.Title = "Value";
        }

        private void txtXMax_TextChanged(object sender, EventArgs e)
        {

        }

        bool IsTextboxDouble(TextBox textBox)
        {
            double parsedValue;
            if (!double.TryParse(textBox.Text, out parsedValue))
            {
                MessageBox.Show("Textbox must contain integer or double value!", "Error");
                return false;
            }
            else return true;
        }

        void UpdateGraphs(string fileName)
        {
            try
            {
                //Declare some stuff
                string seriesOne = txtSeriesOneText.Text;
                string seriesTwo = txtSeriesTwoText.Text;
                string seriesThree = txtSeriesThreeText.Text;
                string seriesFour = txtSeriesFourText.Text;

                List<double> seriesOneValues = new List<double>();
                List<double> seriesTwoValues = new List<double>();
                List<double> seriesThreeValues = new List<double>();
                List<double> seriesFourValues = new List<double>();

                //Clear the series
                chart1.Series.Clear();

                //Setup the series
                chart1.Series.Add("Series1");
                chart1.Series["Series1"].ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.Line;
                chart1.Series["Series1"].Color = Color.Red;
                chart1.Series["Series1"].BorderWidth = 1;
                chart1.Series["Series1"].LegendText = seriesOne;
                chart1.Series["Series1"].YValueType = System.Windows.Forms.DataVisualization.Charting.ChartValueType.Double;
                chart1.Series["Series1"].XValueType = System.Windows.Forms.DataVisualization.Charting.ChartValueType.Double;

                chart1.Series.Add("Series2");
                chart1.Series["Series2"].ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.Line;
                chart1.Series["Series2"].Color = Color.Blue;
                chart1.Series["Series2"].BorderWidth = 1;
                chart1.Series["Series2"].LegendText = seriesTwo;
                chart1.Series["Series2"].YValueType = System.Windows.Forms.DataVisualization.Charting.ChartValueType.Double;
                chart1.Series["Series2"].XValueType = System.Windows.Forms.DataVisualization.Charting.ChartValueType.Double;

                chart1.Series.Add("Series3");
                chart1.Series["Series3"].ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.Line;
                chart1.Series["Series3"].Color = Color.Green;
                chart1.Series["Series3"].BorderWidth = 1;
                chart1.Series["Series3"].LegendText = seriesThree;
                chart1.Series["Series3"].YValueType = System.Windows.Forms.DataVisualization.Charting.ChartValueType.Double;
                chart1.Series["Series3"].XValueType = System.Windows.Forms.DataVisualization.Charting.ChartValueType.Double;

                chart1.Series.Add("Series4");
                chart1.Series["Series4"].ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.Line;
                chart1.Series["Series4"].Color = Color.Purple;
                chart1.Series["Series4"].BorderWidth = 1;
                chart1.Series["Series4"].LegendText = seriesFour;
                chart1.Series["Series4"].YValueType = System.Windows.Forms.DataVisualization.Charting.ChartValueType.Double;
                chart1.Series["Series4"].XValueType = System.Windows.Forms.DataVisualization.Charting.ChartValueType.Double;
                    
                //Work through the log text file and add the values to their respective lists
                var lines = File.ReadAllLines(fileName);

                foreach (var line in lines)
                {
                    if (txtSeriesOneText.Text != "" && line.StartsWith(seriesOne, System.StringComparison.CurrentCultureIgnoreCase) == true)
                    {
                        //Remove the exact string we are looking for from the line to give us onto the numerical value at the end of it
                        string value = line.Remove(0, seriesOne.Length);

                        //Convert the remaining numerical value to a double
                        try
                        {
                            seriesOneValues.Add(Convert.ToDouble(value) * Convert.ToDouble(txtSeriesOneFactor.Text));                       
                        }
                        catch
                        {
                            MessageBox.Show("Reached a non numerical Value: " + Convert.ToString(value), "Error");
                            return;
                        }
                    }
                    else if (txtSeriesTwoText.Text != "" && line.StartsWith(seriesTwo, System.StringComparison.CurrentCultureIgnoreCase) == true)
                    {
                        //Remove the exact string we are looking for from the line to give us onto the numerical value at the end of it
                        string value = line.Remove(0, seriesTwo.Length);

                        //Convert the remaining numerical value to a double
                        try
                        {
                            seriesTwoValues.Add(Convert.ToDouble(value) * Convert.ToDouble(txtSeries2Factor.Text));
                        }
                        catch
                        {
                            MessageBox.Show("Reached a non numerical Value: " + Convert.ToString(value), "Error");
                            return;
                        }
                    }
                    else if (txtSeriesThreeText.Text != "" && line.StartsWith(seriesThree, System.StringComparison.CurrentCultureIgnoreCase) == true)
                    {
                        //Remove the exact string we are looking for from the line to give us onto the numerical value at the end of it
                        string value = line.Remove(0, seriesThree.Length);

                        //Convert the remaining numerical value to a double
                        try
                        {
                            seriesThreeValues.Add(Convert.ToDouble(value) * Convert.ToDouble(txtSeries3Factor.Text));
                        }
                        catch
                        {
                            MessageBox.Show("Reached a non numerical Value: " + Convert.ToString(value), "Error");
                            return;
                        }
                    }
                    else if (txtSeriesFourText.Text != "" && line.StartsWith(seriesFour, System.StringComparison.CurrentCultureIgnoreCase) == true)
                    {
                        //Remove the exact string we are looking for from the line to give us onto the numerical value at the end of it
                        string value = line.Remove(0, seriesFour.Length);

                        //Convert the remaining numerical value to a double
                        try
                        {
                            seriesFourValues.Add(Convert.ToDouble(value) * Convert.ToDouble(txtSeries4Factor.Text));
                        }
                        catch
                        {
                            MessageBox.Show("Reached a non numerical Value: " + Convert.ToString(value), "Error");
                            return;
                        }
                    }

                }

                //Actually graph the stuff!
                AddToChart("Series1", seriesOneValues);
                AddToChart("Series2", seriesTwoValues);
                AddToChart("Series3", seriesThreeValues);
                AddToChart("Series4", seriesFourValues);   
            }
            catch
            {
                MessageBox.Show("Error updating the graphs!", "Error");
            }
        }

        void AddToChart(string series, List<double> seriesValues)
        {
            double[] values = seriesValues.ToArray();

            int sizeOfArray = values.Length;

            for(int x = 0; x < sizeOfArray; x++)
            {
                chart1.Series[series].Points.AddXY(x + 1, values[x]);
            }

            //Update the # of records labels
            if (series == "Series1") lblSeries1Records.Text = "Series 1: " + values.Length;
            else if (series == "Series2") lblSeries2Records.Text = "Series 2: " + values.Length;
            else if (series == "Series3") lblSeries3Records.Text = "Series 3: " + values.Length;
            else if (series == "Series4") lblSeries4Records.Text = "Series 4: " + values.Length;
        }

        private void btnUpdate_Click(object sender, EventArgs e)
        {
            //Basically does the same thing as the load .txt file button except for allowing the user to choose the text file ... it looks for the one previously opened
            try
            {
                BuildChart();
                UpdateGraphs(fileName);
            }
            catch
            {
                MessageBox.Show("There was a problem opening and processing the log file.", "Error");
            }
        }
    }
}
